
#ifndef BSPLIT_COMMAND_VER
#define BSPLIT_COMMAND_VER				1.0.1.0


#ifndef MACRO_STR
#define MACRO_EXP( arg )				#arg
#define MACRO_STR( arg )				MACRO_EXP( arg )
#endif	//	MACRO_STR


#define BSPLIT_FLAG_NO_ERRORS			0x0100
#define BSPLIT_FLAG_QUIETLY				0x0200
#define BSPLIT_FLAG_ALLOW_EMPTY			0x0400
#define BSPLIT_FLAG_ALLOW_ONE_SPLIT		0x0800
#define BSPLIT_FLAG_OVERWRITE			0x1000

#define BSPLIT_FORMAT_BITS				0x003f
#define BSPLIT_FORMAT_2DEC				0x0000
#define BSPLIT_FORMAT_3DEC				0x0001
#define BSPLIT_FORMAT_4DEC				0x0002
#define BSPLIT_FORMAT_2HEX				0x0003
#define BSPLIT_FORMAT_3HEX				0x0004
#define BSPLIT_FORMAT_4HEX				0x0005
#define BSPLIT_FORMAT_2A				0x0006
#define BSPLIT_FORMAT_3A				0x0007
#define BSPLIT_FORMAT_4A				0x0008
#define BSPLIT_FORMAT_UPPERCASE			0x0020



const wchar_t BSplitHelpText[]	= L"Split files into smaller files.\n\
\n\
BSPLIT /D:dir /E /F:n /N:n /O /P /Q /S:size /X:prefix filename... \r\n\
\n\
\t/D:\toutput directory\n\
\t/E\tno error messages\n\
\t/F:\textension numbering format\n\
\t/N:\tnumber of splits\n\
\t/O\toverwrite\n\
\t/P\toutput to source file\u2019s path\n\
\t/Q\tquietly\n\
\t/S:\tsplit size\n\
\t/X:\textension prefix\n\
\n\
BSPLIT divides a file into multiple smaller files.  You can \n\
split at a specified size (/S:), or into a specified number\n\
of files (/N:).  You cannot combine /N: and /S:.\n\
\n\
You must supply a filename.  Wildcards are supported, and\n\
you may have more than one filename.  The output directory\n\
defaults to the current directory.\n";



const wchar_t BSplitSourcePath[]		= L"<<*>>";



struct _BSplit_Counts {
	unsigned int Found;
	unsigned int NotFound;
	unsigned int Succeeded;
	unsigned int Failed;
	unsigned int Skipped;
};



wchar_t* BSplit_FindRootSlash( LPTSTR Directory )
{
	if ( !Directory )
		return NULL;
	else if ( Directory[0] == '\0' )
		return NULL;


	if ( ( Directory[0] == '\\' ) && ( Directory[1] == '\\' ) ) {
		wchar_t *ch		= Directory + 2;

		if ( *ch == '\0' )
			return NULL;

		while ( *ch == '\\' )				//	skip over leading slashes
			ch++;

		if ( *ch == '\0' )
			return NULL;

		while ( *ch && ( *ch != '\\' ) )	//	skip over compuer name
			ch++;

		if ( *ch != '\\' )
			return NULL;

		while ( *ch == '\\' )				//	skip over slash between computer and share names
			ch++;

		if ( *ch == '\0' )
			return NULL;

		while ( *ch && ( *ch != '\\' ) )	//	skip over share name
			ch++;

		if ( *ch != '\\' )
			return NULL;

		while ( ch[1] == '\\' )
			ch++;

		return ch;
	}

	return wcschr( Directory, '\\' );
}



int BSplit_CreateDirRecursive( LPTSTR Dir, LPTSTR RootSlash )
{
	if ( !Dir )
		return -1;
	else if ( Dir[0] == '\0' )
		return -1;


	wchar_t *ch			= NULL;

	int rv				= 0;


	if ( !CreateDirectory( Dir, NULL ) )
		rv = GetLastError();

	if ( rv == ERROR_PATH_NOT_FOUND ) {
		ch = wcsrchr( Dir, '\\' );

		if ( ch && ( ch > RootSlash ) ) {
			*ch = '\0';
			rv = BSplit_CreateDirRecursive( Dir, RootSlash );
			*ch = '\\';

			if ( rv == ERROR_SUCCESS )
				if ( !CreateDirectory( Dir, NULL ) )
					rv = GetLastError();
		}
	}

	return rv;
}



int BSplit_CreateNestedDirectory( LPCTSTR Directory )
{
	if ( !Directory )
		return -1;
	else if ( Directory[0] == '\0' )
		return -1;


	wchar_t dir[MAX_FILENAME_LEN]		= L"";

	wchar_t *rs							= NULL;
	wchar_t *ch							= NULL;


	wcscpy_s( dir, MAX_FILENAME_LEN, Directory );

	for ( wchar_t *ch = dir; *ch; ch++ )
		if ( *ch == '/' )
			*ch = '\\';

	rs = BSplit_FindRootSlash( dir );

	if ( rs ) {
		ch = dir + wcslen( dir ) - 1;
		while ( ( ch > rs ) && ( *ch == '\\' ) ) {		//	snip off any trailing slashes
			*ch = '\0';
			ch--;
		}
	}

	return BSplit_CreateDirRecursive( dir, rs );
}



int BSplit_GetFileSize( LPCTSTR Filename, unsigned __int64 *Size )
{
	if ( !Filename || !Size )
		return -1;
	else if ( Filename[0] == '\0' )
		return -1;


	HANDLE FileHandle			= NULL;
	LARGE_INTEGER FileSize;
	int rv						= 1;


	memset( &FileSize, 0x00, sizeof( LARGE_INTEGER ) );

	FileHandle = CreateFile( Filename, FILE_READ_ATTRIBUTES, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );

	if ( !FileHandle || ( FileHandle == INVALID_HANDLE_VALUE ) )
		return 1;

	if ( GetFileSizeEx( FileHandle, &FileSize ) ) {
		*Size = FileSize.QuadPart;
		rv = 0;
	}

	CloseHandle( FileHandle );
	return rv;
}



int BSplit_B26Str( LPTSTR outString, unsigned int n, unsigned int minlen )
{
	if ( !outString )
		return -1;


	size_t j			= 0;


	outString[j] = '\0';


	while ( n || ( j < minlen ) ) {
		outString[j++] = 'a' + ( n % 26 );
		n = n / 26;
	}

	outString[j] = '\0';
	_wcsrev( outString );
	return 0;
}



int BSplit_MakeSplitName( LPCTSTR Dir, LPCTSTR BaseName, LPCTSTR Prefix, unsigned int n, unsigned int fmt, LPTSTR outFilename )
{
	if ( !BaseName || !outFilename )
		return -1;
	else if ( BaseName[0] == '\0' )
		return 01;


	wchar_t ext[MAX_ARG_LEN]			= L"";
	wchar_t empty[1]					= L"";


	if ( !Dir )
		Dir = empty;


	wcscpy_s( outFilename, MAX_FILENAME_LEN, Dir );
	AppendFilename( outFilename, BaseName, MAX_FILENAME_LEN );
	wcscat_s( outFilename, MAX_FILENAME_LEN, L"." );

	if ( Prefix )
		if ( Prefix[0] )
			wcscat_s( outFilename, MAX_FILENAME_LEN, Prefix );


	switch ( fmt & ( BSPLIT_FORMAT_BITS & ~BSPLIT_FORMAT_UPPERCASE ) ) {

	case BSPLIT_FORMAT_3DEC :
		wsprintf( ext, L"%03u", n );
		break;

	case BSPLIT_FORMAT_4DEC :
		wsprintf( ext, L"%04u", n );
		break;

	case BSPLIT_FORMAT_2HEX :
		wsprintf( ext, L"%02x", n );
		break;

	case BSPLIT_FORMAT_3HEX :
		wsprintf( ext, L"%03x", n );
		break;

	case BSPLIT_FORMAT_4HEX :
		wsprintf( ext, L"%04x", n );
		break;

	case BSPLIT_FORMAT_2A :
		BSplit_B26Str( ext, n, 2 );
		break;

	case BSPLIT_FORMAT_3A :
		BSplit_B26Str( ext, n, 3 );
		break;

	case BSPLIT_FORMAT_4A :
		BSplit_B26Str( ext, n, 4 );
		break;

	default :
	case BSPLIT_FORMAT_2DEC :
		wsprintf( ext, L"%02u", n );
	}

	if ( fmt & BSPLIT_FORMAT_UPPERCASE )
		CharUpperW( ext );

	wcscat_s( outFilename, MAX_FILENAME_LEN, ext );
	return 0;
}



int BSplitFile( LPCTSTR Filename, unsigned int Flags, unsigned __int64 SplitSize, unsigned int NumSplits, LPCTSTR OutDir, LPCTSTR Prefix, _BSplit_Counts *Counts )
{
	if ( !Filename )
		return -1;
	else if ( Filename[0] == '\0' )
		return -1;


	wchar_t CurrentDir[SHORT_BUF_LEN]			= L"";
	wchar_t BaseName[MAX_FILENAME_LEN]			= L"";
	wchar_t OutputFilename[MAX_FILENAME_LEN]	= L"";
	wchar_t ErrorText[SHORT_BUF_LEN]			= L"";

	wchar_t *ch									= NULL;
	BOOL ShowErrors								= ( ( Flags & BSPLIT_FLAG_NO_ERRORS ) == 0 );
	BOOL Quietly								= ( ( Flags & BSPLIT_FLAG_QUIETLY ) != 0 );
	unsigned int Format							= ( Flags & BSPLIT_FORMAT_BITS );

	unsigned __int64 FileSize			= 0;
	unsigned __int64 BytesRemaining		= 0;
	unsigned __int64 RemainingInSplit	= 0;
	unsigned int SplitNum				= 0;
	DWORD XferSize						= 0;
	DWORD Xferred						= 0;
	DWORD CreateMode					= ( Flags & BSPLIT_FLAG_OVERWRITE ) ? CREATE_ALWAYS : CREATE_NEW;
	HANDLE InputHandle					= NULL;
	HANDLE OutputHandle					= NULL;
	BOOL NewSplit						= FALSE;

	const int BufferSize				= 8192;
	BYTE Buffer[BufferSize];

	int err								= 0;
	int rv								= 0;



	if ( !SplitSize && !NumSplits )
		SplitSize = 32 * 1024;

	if ( OutDir )
		if ( OutDir[0] == '\0' )
			OutDir = NULL;

	if ( !OutDir ) {
		GetCurrentDirectory( SHORT_BUF_LEN, CurrentDir );
		if ( CurrentDir[0] == '\0' )
			wcscpy_s( CurrentDir, SHORT_BUF_LEN, L"." );

		OutDir = CurrentDir;
	}
	else if ( !wcscmp( OutDir, BSplitSourcePath ) ) {
		FH_PathPart( (LPTSTR) Filename, CurrentDir );
		OutDir = CurrentDir;
	}
	FH_FilenamePart( (LPTSTR) Filename, BaseName );
	ch = FH_ExtensionPart( BaseName, NULL );
	if ( ch )
		*ch = '\0';


	if ( !QueryIsDirectory( (LPTSTR) OutDir ) ) {
		BSplit_CreateNestedDirectory( OutDir );		//	CreateDirectory( OutDir, NULL );
		err = GetLastError();

		if ( !QueryIsDirectory( (LPTSTR) OutDir ) ) {
			if ( ShowErrors ) {
				GetErrorText( err, ErrorText );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Can\u2019t create directory \"%s\" :\r\n", OutDir );
				Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
			}

			if ( Counts )
				Counts->Failed += 1;

			return 2;
		}
	}


	//	wprintf( L"   *  Filename = '%s'    BaseName = '%s'    OutDir = '%s'\n", Filename, BaseName, OutDir );


	if ( BSplit_GetFileSize( Filename, &FileSize ) ) {
		if ( ShowErrors ) {
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Can\u2019t get file size for \"%s\"\r\n", Filename );
		}

		if ( Counts )
			Counts->Failed += 1;

		return 2;
	}

	if ( ( FileSize == 0 ) && ( ( Flags & BSPLIT_FLAG_ALLOW_EMPTY ) == 0 ) ) {
		if ( !Quietly )
			Printf( L"Skipping file \"%s\" because it is empty.\r\n", Filename );

		if ( Counts )
			Counts->Skipped += 1;

		return 0;
	}


	if ( !SplitSize && NumSplits ) {
		unsigned __int64 ss		= FileSize / NumSplits;
		unsigned __int64 r		= FileSize % ss;

		if ( r )
			ss++;

		SplitSize = ss;
	}

	if ( !NumSplits ) {
		NumSplits = (unsigned int) ( FileSize / SplitSize );
		if ( FileSize % SplitSize )
			NumSplits++;
	}

	if ( ( NumSplits == 1 ) && ( ( Flags & BSPLIT_FLAG_ALLOW_ONE_SPLIT ) == 0 ) ) {
		if ( !Quietly )
			Printf( L"Skipping file \"%s\" because it would only yield one split.\r\n", Filename );

		if ( Counts )
			Counts->Skipped += 1;

		return 0;
	}

	
	//	wprintf( L"   *  FileSize = %I64u    SplitSize = %I64u    NumSplits = %I64u\n", FileSize, SplitSize, NumSplits );


	if ( NumSplits ) {

		//		Precheck for name collisions.  We don't want to create 999 output files
		//		and then bump into an existing filename on number 1000, if we can
		//		avoid it.

		for ( unsigned int n = 0; n < NumSplits; n++ ) {
			BOOL Exists		= FALSE;


			BSplit_MakeSplitName( OutDir, BaseName, Prefix, n, Format, OutputFilename );

			if ( Flags & BSPLIT_FLAG_OVERWRITE ) {
				if ( QueryIsDirectory( OutputFilename ) )
					Exists = TRUE;
			}
			else {
				if ( QueryIsFileOrDirectory( OutputFilename ) )
					Exists = TRUE;
			}

			if ( Exists ) {
				if ( ShowErrors )
					Qprintf( ERRHANDLE, L"Skipping file \"%s\" because \"%s\" already exists.\r\n", Filename, OutputFilename );

				if ( Counts )
					Counts->Failed += 1;

				return 2;
			}
		}
	}


	/*
	if ( ( FileSize == 0 ) && ( Flags & BSPLIT_FLAG_ALLOW_EMPTY ) ) {

		//		Hack to allow creating empty files.  I don't know why
		//		you would ever want this!

		BSplit_MakeSplitName( OutDir, BaseName, Prefix, SplitNum, Format, OutputFilename );
				
		if ( !Quietly )
			Printf( L"Creating file \"%s\"\r\n", OutputFilename );

		OutputHandle = CreateFile( OutputFilename, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL, CreateMode, FILE_ATTRIBUTE_NORMAL, NULL );
		if ( OutputHandle == INVALID_HANDLE_VALUE ) {
			if ( ShowErrors ) {
				GetLastErrorText( ErrorText );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Error creating file \"%s\" :\r\n", OutputFilename );
				Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
			}

			goto Abort;
		}

		if ( !CloseHandle( OutputHandle ) ) {
			if ( ShowErrors ) {
				GetLastErrorText( ErrorText );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Error closing file \"%s\" :\r\n", OutputFilename );
				Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
			}

			goto Abort;
		}

		if ( Counts )
			Counts->Succeeded += 1;

		return 0;
	}
	*/


	InputHandle = CreateFile( Filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );
	if ( InputHandle == INVALID_HANDLE_VALUE ) {
		if ( ShowErrors ) {
			GetLastErrorText( ErrorText );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Error opening \"%s\" :\r\n", Filename );
			Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
		}

		rv = 2;
	}
	else {
		//	wprintf( L"   *  Opened input file '%s'.\n", Filename );

		BytesRemaining = FileSize;
		NewSplit = TRUE;

		while ( BytesRemaining ) {

			if ( NewSplit ) {
				BSplit_MakeSplitName( OutDir, BaseName, Prefix, SplitNum, Format, OutputFilename );
				
				if ( !Quietly )
					Printf( L"Creating file \"%s\"\r\n", OutputFilename );

				OutputHandle = CreateFile( OutputFilename, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL, CreateMode, FILE_ATTRIBUTE_NORMAL, NULL );
				if ( OutputHandle == INVALID_HANDLE_VALUE ) {
					if ( ShowErrors ) {
						GetLastErrorText( ErrorText );
						DisplayErrorHeader();
						Qprintf( ERRHANDLE, L"Error creating file \"%s\" :\r\n", OutputFilename );
						Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
					}

					goto Abort;
				}

				RemainingInSplit = ( BytesRemaining > SplitSize ) ? SplitSize : BytesRemaining;
				NewSplit = FALSE;
			}

			while ( RemainingInSplit ) {

				XferSize = ( RemainingInSplit > (size_t) BufferSize ) ? BufferSize : (DWORD) RemainingInSplit;

				if ( !ReadFile( InputHandle, Buffer, XferSize, &Xferred, NULL ) ) {
					if ( ShowErrors ) {
						GetLastErrorText( ErrorText );
						DisplayErrorHeader();
						Qprintf( ERRHANDLE, L"Error reading file \"%s\" :\r\n", Filename );
						Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
					}

					goto Abort;
				}

				if ( !WriteFile( OutputHandle, Buffer, Xferred, &Xferred, NULL ) ) {
					if ( ShowErrors ) {
						GetLastErrorText( ErrorText );
						DisplayErrorHeader();
						Qprintf( ERRHANDLE, L"Error writing file \"%s\" :\r\n", OutputFilename );
						Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
					}

					goto Abort;
				}

				BytesRemaining		-= Xferred;
				RemainingInSplit	-= Xferred;
			}

			if ( !CloseHandle( OutputHandle ) ) {
				if ( ShowErrors ) {
					GetLastErrorText( ErrorText );
					DisplayErrorHeader();
					Qprintf( ERRHANDLE, L"Error closing file \"%s\" :\r\n", OutputFilename );
					Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
				}

				goto Abort;
			}

			OutputHandle = NULL;
			NewSplit = TRUE;
			SplitNum++;
		}

		if ( !CloseHandle( InputHandle ) ) {
			if ( ShowErrors ) {
				GetLastErrorText( ErrorText );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Error closing file \"%s\" :\r\n", Filename );
				Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
			}

			goto Abort;
		}
	}

	if ( Counts ) {
		if ( rv )
			Counts->Failed += 1;
		else
			Counts->Succeeded += 1;

	}

	return rv;


Abort :

	CloseHandle( OutputHandle );
	CloseHandle( InputHandle );

	if ( Counts )
		Counts->Failed += 1;

	return 2;
}



int BSplitMatchingFiles( LPCTSTR Wildspec, unsigned int Flags, unsigned __int64 SplitSize, unsigned int NumSplits, LPCTSTR OutDir, LPCTSTR Prefix, _BSplit_Counts *Counts )
{
	if ( !Wildspec )
		return -1;


	wchar_t FullName[MAX_FILENAME_LEN]		= L"";
	wchar_t ErrorText[SHORT_BUF_LEN]		= L"";

	HANDLE FindHandle		= NULL;
	WIN32_FIND_DATA FoundItem;
	unsigned int NumFound	= 0;

	int rv					= 0;


	FindHandle = FindFirstFile( Wildspec, &FoundItem );

	if ( FindHandle == INVALID_HANDLE_VALUE ) {
		rv = GetLastError();

		if ( ( Flags & BSPLIT_FLAG_NO_ERRORS ) == 0 ) {
			GetErrorText( rv, ErrorText );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Error matching \"%s\" :\r\n", Wildspec );
			Qprintf( ERRHANDLE, L"   %s\r\n", ErrorText );
		}

		if ( Counts )
			Counts->NotFound += 1;

		return rv;
	}

	do {
		if ( FoundItem.dwFileAttributes & ( FILE_ATTRIBUTE_DIRECTORY | FILE_ATTRIBUTE_DEVICE ) )
			continue;

		if ( !wcscmp( FoundItem.cFileName, L"." ) || !wcscmp( FoundItem.cFileName, L".." ) )
			continue;

		FullName[0] = '\0';
		FH_PathPart( (LPTSTR) Wildspec, FullName );
		AppendFilename( FullName, FoundItem.cFileName, MAX_FILENAME_LEN );
		NumFound++;

		if ( Counts )
			Counts->Found += 1;


		if ( BSplitFile( FullName, Flags, SplitSize, NumSplits, OutDir, Prefix, Counts ) )
			rv = 2;
	}
	while ( FindNextFile( FindHandle, &FoundItem ) );

	FindClose( FindHandle );

	if ( NumFound )
		return rv;
	

	if ( ( Flags & BSPLIT_FLAG_NO_ERRORS ) == 0 ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Error matching \"%s\" :\r\n", Wildspec );
		Qprintf( ERRHANDLE, L"   No matching files were found.\r\n" );
	}

	if ( Counts )
		Counts->NotFound += 1;

	return ERROR_FILE_NOT_FOUND;
}



DLLExports int WINAPI bsplit( LPTSTR inString )
{
	if ( !inString )
		return 1;


	wchar_t Filename[MAX_FILENAME_LEN]		= L"";
	wchar_t OutDir[MAX_FILENAME_LEN]		= L"";
	wchar_t Prefix[SHORT_BUF_LEN]			= L"";

	int Flags					= 0;
	unsigned int Format			= BSPLIT_FORMAT_3DEC;

	int NumFilenames			= 0;
	unsigned __int64 SplitSize	= 0;
	unsigned int NumSplits		= 0;
	BOOL ShowSummary			= FALSE;

	_BSplit_Counts Counts;

	wchar_t *ch					= NULL;
	int trv						= 0;
	int rv						= 0;

	
	ParseArgs( inString, PARSE_BREAK_SPACES | PARSE_SLASHES_KLUDGE );


	for ( int i = 0; i < numargs; i++ ) {
		if ( !wcsncmp( arg[i], L"/?", 2 ) )
			return ShowCmdHelp( BSplitHelpText, ( arg[i][2] == '?' ), __FILE__, MACRO_STR( BSPLIT_COMMAND_VER ) );
		else if ( !wcscmp( arg[i], L"/1" ) )
			Flags |= BSPLIT_FLAG_ALLOW_ONE_SPLIT;
		else if ( !_wcsnicmp( arg[i], L"/D", 2 ) ) {
			ch = arg[i] + 2;
			if ( IsColonOrEquals( *ch ) )
				ch++;

			if ( *ch ) {
				FH_Truename( ch, OutDir );
				if ( OutDir[0] == '\0' ) {
					CharUpperW( arg[i] );
					DisplayErrorHeader();
					Qprintf( ERRHANDLE, L"Bad directory name: \"%s\"\r\n", arg[i] );
					return 1;
				}
			}
			else {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Missing directory: \"%s\"\r\n", arg[i] );
				return 1;
			}
		}
		else if ( !_wcsicmp( arg[i], L"/E" ) )
			Flags |= BSPLIT_FLAG_NO_ERRORS;
		else if ( !_wcsnicmp( arg[i], L"/F", 2 ) ) {
			ch = arg[i] + 2;
			if ( IsColonOrEquals( *ch ) )
				ch++;

			if ( ParseInt( ch, &Format, NULL ) ) {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Bad format value: \"%s\"\r\n", arg[i] );
				return 1;
			}

			Format &= BSPLIT_FORMAT_BITS;
		}
		else if ( !_wcsnicmp( arg[i], L"/N", 2 ) ) {
			ch = arg[i] + 2;
			if ( IsColonOrEquals( *ch ) )
				ch++;

			if ( ParseInt( ch, &NumSplits, NULL ) ) {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Bad NumSplits value: \"%s\"\r\n", arg[i] );
				return 1;
			}

			if ( NumSplits < 2 ) {
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"NumSplits must be greater than 1: %u\r\n", NumSplits );
				return 1;
			}
		}
		else if ( !_wcsicmp( arg[i], L"/O" ) )
			Flags |= BSPLIT_FLAG_OVERWRITE;
		else if ( !_wcsicmp( arg[i], L"/P" ) )
			wcscpy_s( OutDir, MAX_FILENAME_LEN, BSplitSourcePath );
		else if ( !_wcsicmp( arg[i], L"/Q" ) )
			Flags |= BSPLIT_FLAG_QUIETLY;
		else if ( !_wcsnicmp( arg[i], L"/S", 2 ) ) {
			ch = arg[i] + 2;
			if ( IsColonOrEquals( *ch ) )
				ch++;

			if ( *ch == '\0' ) {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Missing size: \"%s\"\r\n", arg[i] );
				return 1;
			}

			if ( ParseInt64( ch, &SplitSize, &ch ) ) {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Bad SplitSize value: \"%s\"\r\n", arg[i] );
				return 1;
			}

			if ( *ch ) {
				switch ( *ch ) {

				case 'b' :
				case 'B' :
					break;

				case 'k' :
					SplitSize *= 1000;
					break;

				case 'K' :
					SplitSize *= 1024;
					break;

				case 'm' :
					SplitSize *= 1000000;
					break;

				case 'M' :
					SplitSize *= ( 1024 * 1024 );
					break;

				case 'g' :
					SplitSize *= 1000000000L;
					break;

				case 'G' :
					SplitSize *= ( 1024 * 1024 * 1024 );
					break;

				default :
					CharUpperW( arg[i] );
					DisplayErrorHeader();
					Qprintf( ERRHANDLE, L"Bad size suffix: \"%s\"\r\n", arg[i] );
					return 1;
				}
			}

			if ( !SplitSize ) {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Split size must be nonzero: \"%s\"\r\n", arg[i] );
				return 1;
			}
		}
		else if ( !_wcsicmp( arg[i], L"/U" ) )
			ShowSummary = TRUE;
		else if ( !_wcsnicmp( arg[i], L"/X", 2 ) ) {
			ch = arg[i] + 2;
			if ( IsColonOrEquals( *ch ) )
				ch++;

			while ( *ch == '.' )
				ch++;

			if ( wcslen( ch ) >= SHORT_BUF_LEN ) {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Prefix too long: \"%s\"\r\n", arg[i] );
			}

			Prefix[0] = '\0';
			wcscpy_s( Prefix, SHORT_BUF_LEN, ch );
		}
		else if ( arg[i][0] == '/' ) {
			CharUpperW( arg[i] );
			Qprintf( ERRHANDLE, L"Unknown option: \"%s\"\r\n", arg[i] );
			return 1;
		}
		else
			NumFilenames++;
	}


	if ( !NumFilenames ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Missing filename\r\n" );
		return 1;
	}

	if ( SplitSize && NumSplits ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Cannot combine /N: and /S:\r\n" );
		return 1;
	}

	if ( !SplitSize && !NumSplits )
		SplitSize = 32 * 1024;


	rv = 0;
	memset( &Counts, 0x00, sizeof( _BSplit_Counts ) );


	for ( int i = 0; i < numargs; i++ ) {
		if ( arglen[i] ) {
			if ( arg[i][0] != '/' ) {
				FH_Truename( arg[i], Filename, MAX_FILENAME_LEN, TRUENAME_FLAG_HOME_PATHS );

				if ( trv = BSplitMatchingFiles( Filename, Flags | Format, SplitSize, NumSplits, OutDir, Prefix, &Counts ) )
					rv = 2;
			}
		}
	}

	if ( ShowSummary )
		Printf( L"\n   Found: %u    Not found: %u    Succeeded: %u    Failed: %u    Skipped: %u\r\n", Counts.Found, Counts.NotFound, Counts.Succeeded, Counts.Failed, Counts.Skipped );

	return rv;
}


#endif	//	BSPLIT_COMMAND_VER


/*

	1.0.0.0		2023-08-03
				First release.

	1.0.1.0		2023-08-07
				Added support for wildcards.  Added /U summary.

*/
